#ifndef GUIDOFONT_H
#define GUIDOFONT_H
/*
  GUIDO Library
  Copyright (C) 2014	Grame

  This Source Code Form is subject to the terms of the Mozilla Public
  License, v. 2.0. If a copy of the MPL was not distributed with this
  file, You can obtain one at http://mozilla.org/MPL/2.0/.

  Grame Research Laboratory, 11, cours de Verdun Gensoul 69002 Lyon - France
  research@grame.fr

*/

#include "VGFont.h"

class GuidoFont : public VGFont
{
    public:
        /*!
		 *  \brief Construct guido font with precaculate metrics.
		 *
         */
		GuidoFont(const char * name, int size, int properties);
        virtual ~GuidoFont();

        const char *	GetName() const;
        int 			GetSize() const;
        int				GetProperties() const;

        void	GetExtent( const char * s, int inCharCount, float * outWidth, float * outHeight, VGDevice * context ) const;
        void	GetExtent( unsigned char c, float * outWidth, float * outHeight, VGDevice * context ) const;

    private:
        std::string name;
		int size;
        int properties;
};
//---------------------------------------------
// Font metrics generated for the Guido2 font
// Font size is 200
//---------------------------------------------

const int kGuidoFontSize = 200;

static const int kGuidoFontWidth[] = {
    100,	// 0
    100,	// 1
    100,	// 2
    100,	// 3
    100,	// 4
    100,	// 5
    100,	// 6
    100,	// 7
    100,	// 8
    80,	// 9
    100,	// 10
    100,	// 11
    100,	// 12
    100,	// 13
    100,	// 14
    100,	// 15
    100,	// 16
    100,	// 17
    100,	// 18
    100,	// 19
    100,	// 20
    100,	// 21
    100,	// 22
    100,	// 23
    100,	// 24
    100,	// 25
    100,	// 26
    100,	// 27
    100,	// 28
    100,	// 29
    100,	// 30
    100,	// 31
    20,	// 32
    42,	// 33
    28,	// 34
    43,	// 35
    29,	// 36
    103,	// 37
    123,	// 38
    189,	// 39
    33,	// 40
    33,	// 41
    60,	// 42
    60,	// 43
    36,	// 44
    60,	// 45
    19,	// 46
    108,	// 47
    72,	// 48
    50,	// 49
    68,	// 50
    65,	// 51
    66,	// 52
    60,	// 53
    67,	// 54
    68,	// 55
    66,	// 56
    67,	// 57
    72,	// 58
    83,	// 59
    41,	// 60
    200,	// 61
    76,	// 62
    136,	// 63
    65,	// 64
    30,	// 65
    134,	// 66
    84,	// 67
    63,	// 68
    60,	// 69
    150,	// 70
    41,	// 71
    80,	// 72
    41,	// 73
    54,	// 74
    56,	// 75
    82,	// 76
    117,	// 77
    20,	// 78
    71,	// 79
    147,	// 80
    60,	// 81
    60,	// 82
    106,	// 83
    105,	// 84
    112,	// 85
    52,	// 86
    128,	// 87
    60,	// 88
    200,	// 89
    117,	// 90
    52,	// 91
    52,	// 92
    89,	// 93
    53,	// 94
    100,	// 95
    122,	// 96
    50,	// 97
    40,	// 98
    84,	// 99
    149,	// 100
    4,	// 101
    89,	// 102
    31,	// 103
    5,	// 104
    30,	// 105
    109,	// 106
    110,	// 107
    91,	// 108
    121,	// 109
    46,	// 110
    28,	// 111
    77,	// 112
    60,	// 113
    112,	// 114
    227,	// 115
    52,	// 116
    112,	// 117
    53,	// 118
    81,	// 119
    60,	// 120
    150,	// 121
    52,	// 122
    21,	// 123
    64,	// 124
    91,	// 125
    69,	// 126
    100,	// 127
    100,	// 128
    100,	// 129
    100,	// 130
    100,	// 131
    100,	// 132
    100,	// 133
    100,	// 134
    100,	// 135
    100,	// 136
    100,	// 137
    100,	// 138
    100,	// 139
    100,	// 140
    100,	// 141
    100,	// 142
    100,	// 143
    100,	// 144
    100,	// 145
    100,	// 146
    100,	// 147
    100,	// 148
    100,	// 149
    100,	// 150
    100,	// 151
    100,	// 152
    100,	// 153
    100,	// 154
    100,	// 155
    0,	// 156
    100,	// 157
    100,	// 158
    100,	// 159
    63,	// 160
    40,	// 161
    50,	// 162
    49,	// 163
    240,	// 164
    51,	// 165
    46,	// 166
    41,	// 167
    56,	// 168
    35,	// 169
    43,	// 170
    25,	// 171
    82,	// 172
    0,	// 173
    78,	// 174
    4,	// 175
    158,	// 176
    122,	// 177
    23,	// 178
    56,	// 179
    53,	// 180
    170,	// 181
    47,	// 182
    91,	// 183
    105,	// 184
    105,	// 185
    47,	// 186
    170,	// 187
    69,	// 188
    121,	// 189
    113,	// 190
    60,	// 191
    119,	// 192
    139,	// 193
    147,	// 194
    289,	// 195
    33,	// 196
    33,	// 197
    81,	// 198
    55,	// 199
    289,	// 200
    60,	// 201
    109,	// 202
    56,	// 203
    63,	// 204
    40,	// 205
    88,	// 206
    6,	// 207
    6,	// 208
    95,	// 209
    135,	// 210
    113,	// 211
    154,	// 212
    123,	// 213
    60,	// 214
    60,	// 215
    118,	// 216
    60,	// 217
    5,	// 218
    60,	// 219
    60,	// 220
    109,	// 221
    57,	// 222
    149,	// 223
    20,	// 224
    34,	// 225
    35,	// 226
    87,	// 227
    20,	// 228
    105,	// 229
    89,	// 230
    63,	// 231
    60,	// 232
    60,	// 233
    60,	// 234
    60,	// 235
    60,	// 236
    60,	// 237
    33,	// 238
    33,	// 239
    46,	// 240
    46,	// 241
    100,	// 242
    100,	// 243
    100,	// 244
    100,	// 245
    100,	// 246
    100,	// 247
    100,	// 248
    100,	// 249
    100,	// 250
    100,	// 251
    100,	// 252
    100,	// 253
    100,	// 254
    100,	// 255
    137,	// 256
};

static const int kGuidoFontHeight[] = {
    499,	// 0
    499,	// 1
    499,	// 2
    499,	// 3
    499,	// 4
    499,	// 5
    499,	// 6
    499,	// 7
    499,	// 8
    499,	// 9
    499,	// 10
    499,	// 11
    499,	// 12
    499,	// 13
    499,	// 14
    499,	// 15
    499,	// 16
    499,	// 17
    499,	// 18
    499,	// 19
    499,	// 20
    499,	// 21
    499,	// 22
    499,	// 23
    499,	// 24
    499,	// 25
    499,	// 26
    499,	// 27
    499,	// 28
    499,	// 29
    499,	// 30
    499,	// 31
    499,	// 32
    499,	// 33
    499,	// 34
    499,	// 35
    499,	// 36
    499,	// 37
    499,	// 38
    499,	// 39
    499,	// 40
    499,	// 41
    499,	// 42
    499,	// 43
    499,	// 44
    499,	// 45
    499,	// 46
    499,	// 47
    499,	// 48
    499,	// 49
    499,	// 50
    499,	// 51
    499,	// 52
    499,	// 53
    499,	// 54
    499,	// 55
    499,	// 56
    499,	// 57
    499,	// 58
    499,	// 59
    499,	// 60
    499,	// 61
    499,	// 62
    499,	// 63
    499,	// 64
    499,	// 65
    499,	// 66
    499,	// 67
    499,	// 68
    499,	// 69
    499,	// 70
    499,	// 71
    499,	// 72
    499,	// 73
    499,	// 74
    499,	// 75
    499,	// 76
    499,	// 77
    499,	// 78
    499,	// 79
    499,	// 80
    499,	// 81
    499,	// 82
    499,	// 83
    499,	// 84
    499,	// 85
    499,	// 86
    499,	// 87
    499,	// 88
    499,	// 89
    499,	// 90
    499,	// 91
    499,	// 92
    499,	// 93
    499,	// 94
    499,	// 95
    499,	// 96
    499,	// 97
    499,	// 98
    499,	// 99
    499,	// 100
    499,	// 101
    499,	// 102
    499,	// 103
    499,	// 104
    499,	// 105
    499,	// 106
    499,	// 107
    499,	// 108
    499,	// 109
    499,	// 110
    499,	// 111
    499,	// 112
    499,	// 113
    499,	// 114
    499,	// 115
    499,	// 116
    499,	// 117
    499,	// 118
    499,	// 119
    499,	// 120
    499,	// 121
    499,	// 122
    499,	// 123
    499,	// 124
    499,	// 125
    499,	// 126
    499,	// 127
    499,	// 128
    499,	// 129
    499,	// 130
    499,	// 131
    499,	// 132
    499,	// 133
    499,	// 134
    499,	// 135
    499,	// 136
    499,	// 137
    499,	// 138
    499,	// 139
    499,	// 140
    499,	// 141
    499,	// 142
    499,	// 143
    499,	// 144
    499,	// 145
    499,	// 146
    499,	// 147
    499,	// 148
    499,	// 149
    499,	// 150
    499,	// 151
    499,	// 152
    499,	// 153
    499,	// 154
    499,	// 155
    499,	// 156
    499,	// 157
    499,	// 158
    499,	// 159
    499,	// 160
    499,	// 161
    499,	// 162
    499,	// 163
    499,	// 164
    499,	// 165
    499,	// 166
    499,	// 167
    499,	// 168
    499,	// 169
    499,	// 170
    499,	// 171
    499,	// 172
    499,	// 173
    499,	// 174
    499,	// 175
    499,	// 176
    499,	// 177
    499,	// 178
    499,	// 179
    499,	// 180
    499,	// 181
    499,	// 182
    499,	// 183
    499,	// 184
    499,	// 185
    499,	// 186
    499,	// 187
    499,	// 188
    499,	// 189
    499,	// 190
    499,	// 191
    499,	// 192
    499,	// 193
    499,	// 194
    499,	// 195
    499,	// 196
    499,	// 197
    499,	// 198
    499,	// 199
    499,	// 200
    499,	// 201
    499,	// 202
    499,	// 203
    499,	// 204
    499,	// 205
    499,	// 206
    499,	// 207
    499,	// 208
    499,	// 209
    499,	// 210
    499,	// 211
    499,	// 212
    499,	// 213
    499,	// 214
    499,	// 215
    499,	// 216
    499,	// 217
    499,	// 218
    499,	// 219
    499,	// 220
    499,	// 221
    499,	// 222
    499,	// 223
    499,	// 224
    499,	// 225
    499,	// 226
    499,	// 227
    499,	// 228
    499,	// 229
    499,	// 230
    499,	// 231
    499,	// 232
    499,	// 233
    499,	// 234
    499,	// 235
    499,	// 236
    499,	// 237
    499,	// 238
    499,	// 239
    499,	// 240
    499,	// 241
    499,	// 242
    499,	// 243
    499,	// 244
    499,	// 245
    499,	// 246
    499,	// 247
    499,	// 248
    499,	// 249
    499,	// 250
    499,	// 251
    499,	// 252
    499,	// 253
    499,	// 254
    499,	// 255
    499,	// 256
};
#endif // GUIDOFONT_H
